/*
   Recursive-descent integer arithmetic calculator
   according to the EBNF:

   <e> -> <t> {+ <t>}
   <t> -> <f> {* <f>}
   <f> -> ( <e> ) | Number

   Inputs a line of text from stdin.
   Outputs "Error" or the result.
*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

char token;

int exp(void);
int term(void);
int factor(void);

/* report error and die */
void error(void)
{ fprintf(stderr,"Error\n");
  exit(1);
}

/* match input token, read next token */
void match( char expectedToken)
{ if (token==expectedToken) token = getchar();
  else error();
}

/* calculator driver program */
main()
{ int result;
  token = getchar();
  result = exp();
  if (token=='\n') printf("Result = %d\n",result);
  else error();
  return 0;
}

/* process an expression */
int exp(void)		
{ int temp = term();
  while (token=='+')
    switch (token) 
    { case '+': match('+'); temp += term(); 
    }
  return temp;
}

/* process a term */
int term(void)		
{ int temp = factor();
  while (token=='*')
  { match('*'); temp *= factor(); }
  return temp;
}

/* process a factor */
int factor(void)	
{ int temp;
  if (token=='(')
  { match('('); temp = exp(); match(')'); }
  else if (isdigit(token))
  { ungetc(token,stdin);
    scanf("%d",&temp);
    token = getchar();
  }
  else error();
  return temp;
}
